@@ -8,9 +8,66 @@ use YAML::Tiny;
 
 # ABSTRACT: Validation framework that can be configured with YAML files
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 our $errstr  = '';
 
+=head1 SYNOPSIS
+
+Perhaps a little code snippet.
+
+    use Data::Validate::WithYAML;
+
+    my $foo = Data::Validate::WithYAML->new( 'test.yml' );
+    my %map = (
+        name     => 'Test Person',
+        password => 'xasdfjakslr453$',
+        plz      => 64569,
+        word     => 'Herr',
+        age      => 55,
+    );
+    
+    for my $field ( keys %map ){
+        print "ok: ",$map{$field},"\n" if $foo->check( $field, $map{$field} );
+    }
+
+data.yml
+
+  ---
+  step1:
+      name:
+          type: required
+          length: 8,122
+      password:
+          type: required
+          length: 10,
+      plz:
+          regex: ^\d{4,5}$
+          type: optional
+      word:
+          enum:
+              - Herr
+              - Frau
+              - Firma
+      age:
+          type: required
+          min: 18
+          max: 65
+  
+
+=head1 METHODS
+
+=head2 new
+
+  my $foo = Data::Validate::WithYAML->new( 'filename' );
+  my $foo = Data::Validate::WithYAML->new(
+      'filename',
+      allow_subs => 1,
+      no_steps   => 1,
+  );
+
+creates a new object.
+
+=cut
 
 sub new{
     my ($class,$filename,%args) = @_;
@@ -21,8 +78,9 @@ sub new{
     $self->{__optional__} = {};
     $self->{__required__} = {};
     
-    $self->_yaml_config($filename) or return undef;
-    $self->_allow_subs($args{allow_subs});
+    $self->_allow_subs( $args{allow_subs} );
+    $self->_no_steps( $args{no_steps} );
+    $self->_yaml_config( $filename ) or return undef;
     
     return $self;
 }
@@ -37,6 +95,17 @@ sub _required {
     $self->{__required__};
 }
 
+sub _no_steps {
+    my ($self, $no_steps) = @_;
+    $self->{__no_steps__} = $no_steps if @_ == 2;;
+    $self->{__no_steps__};
+}
+
+=head2 set_optional
+
+This method makes a field optional if it was required
+
+=cut
 
 sub set_optional {
     my ($self,$field) = @_;
@@ -47,6 +116,11 @@ sub set_optional {
     }
 }
 
+=head2 set_required
+
+This method makes a field required if it was optional
+
+=cut
 
 sub set_required {
     my ($self,$field) = @_;
@@ -57,10 +131,32 @@ sub set_required {
     }
 }
 
+=head2 validate
+
+This subroutine validates one form. You have to pass the form name (key in the
+config file), a hash with fieldnames and its values
+
+    my %fields = (
+        username => $cgi->param('user'),
+        passwort => $password,
+    );
+    $foo->validate( 'step1', %fields );
+
+=cut
 
 sub validate{
-    my ($self,$part,%hash) = @_;
-    
+    my $self = shift;
+
+    my ($part, %hash);
+
+    if ( @_ && @_ % 2 == 0 ) {
+        %hash = @_;
+        $part = '';
+    }
+    else {
+        ($part, %hash) = @_;
+    }
+
     my @fieldnames  = $self->fieldnames( $part );
     
     my %errors;
@@ -84,15 +180,15 @@ sub validate{
                 next;
             }
             
-            my $depens_on_value = $hash{$depends_on};
-            my $cases = $fields{$name}->{case} || {};
+            my $depends_on_value = $hash{$depends_on};
+            my $cases            = $fields{$name}->{case} || {};
             
             #if ( !$cases->{$value} ) {
             #    $errors{$name} = $self->message( $name );
             #    next;
             #}
             
-            $fields{$name} = $cases->{$depens_on_value} if $cases->{$depens_on_value};
+            $fields{$name} = $cases->{$depends_on_value} if $cases->{$depends_on_value};
         }
         
         $fields{$name}->{type} ||= 'optional';
@@ -105,13 +201,26 @@ sub validate{
     return %errors;
 }
 
+=head2 fieldnames
+
+=cut
 
 sub fieldnames{
-    my ($self,$step,%options) = @_;
-    
+    my $self = shift;
+
+    my ($step, %options);
+
+    if ( @_ && @_ % 2 == 0 ) {
+        %options = @_;
+        $step = '';
+    }
+    else {
+        ($step, %options) = @_;
+    }
+
     my @names;
     if( defined $step ){
-        @names = @{ $self->{fieldnames}->{$step} };
+        @names = @{ $self->{fieldnames}->{$step} || [] };
     }
     else{
         for my $step ( keys %{ $self->{fieldnames} } ){
@@ -131,12 +240,22 @@ sub fieldnames{
     return @names;
 }
 
+=head2 errstr
+
+=cut
 
 sub errstr{
     my ($self) = @_;
     return $errstr;
 }
 
+=head2 message
+
+returns the message if specified in YAML
+
+  $obj->message( 'fieldname' );
+
+=cut
 
 sub message {
     my ($self,$field) = @_;
@@ -151,6 +270,19 @@ sub message {
     $message;
 }
 
+=head2 check_list
+
+  $obj->check_list('fieldname',['value','value2']);
+
+Checks if the values match the validation criteria. Returns an arrayref with
+checkresults:
+
+    [
+        1,
+        0,
+    ] 
+
+=cut
 
 sub check_list {
     my ($self,$field,$values) = @_;
@@ -166,6 +298,14 @@ sub check_list {
     return \@results;
 }
 
+=head2 check
+
+  $obj->check('fieldname','value');
+
+checks if a value is valid. returns 1 if the value is valid, otherwise it
+returns 0.
+
+=cut
 
 sub check{
     my ($self,$field,$value,$definition) = @_;
@@ -229,21 +369,13 @@ sub _yaml_config{
         $self->{config} = YAML::Tiny->read( $file ) or 
                 (($errstr = YAML::Tiny->errstr()) && return undef);
 
-        for my $section(keys %{$self->{config}->[0]}){
-            my $sec_hash = $self->{config}->[0]->{$section};
-            for my $field(keys %$sec_hash){
-                if(exists $sec_hash->{$field}->{type} and 
-                          $sec_hash->{$field}->{type} eq 'required'){
-                    $self->_required->{$field} = $sec_hash->{$field};
-                    if( exists $self->_optional->{$field} ){
-                        delete $self->_optional->{$field};
-                    }
-                }
-                elsif( not exists $self->_required->{$field} ){
-                    $self->_optional->{$field} = $sec_hash->{$field};
-                }
-
-                push @{$self->{fieldnames}->{$section}}, $field;
+        if ( $self->_no_steps ) {
+            $self->_add_fields( $self->{config}->[0], '' );
+        }
+        else {
+            for my $section(keys %{$self->{config}->[0]}){
+                my $sec_hash = $self->{config}->[0]->{$section};
+                $self->_add_fields( $sec_hash, $section );
             }
         }
     }
@@ -255,6 +387,26 @@ sub _yaml_config{
     return $self->{config};
 }
 
+sub _add_fields {
+    my ($self, $data, $section) = @_;
+
+    for my $field( keys %$data ){
+        if(exists $data->{$field}->{type} and
+                  $data->{$field}->{type} eq 'required'){
+            $self->_required->{$field} = $data->{$field};
+
+            if( exists $self->_optional->{$field} ){
+                delete $self->_optional->{$field};
+            }
+        }
+        elsif( not exists $self->_required->{$field} ){
+            $self->_optional->{$field} = $data->{$field};
+        }
+
+        push @{$self->{fieldnames}->{$section}}, $field;
+    }
+}
+
 sub _min{
     my ($value,$min) = @_;
     return $value >= $min;
@@ -311,178 +463,4 @@ sub _allow_subs {
     $self->{__allow_subs};
 }
 
-
-1; # End of Data::Validate::WithYAML
-
-__END__
-=pod
-
-=head1 NAME
-
-Data::Validate::WithYAML - Validation framework that can be configured with YAML files
-
-=head1 VERSION
-
-version 0.09
-
-=head1 SYNOPSIS
-
-Perhaps a little code snippet.
-
-    use Data::Validate::WithYAML;
-
-    my $foo = Data::Validate::WithYAML->new( 'test.yml' );
-    my %map = (
-        name     => 'Test Person',
-        password => 'xasdfjakslr453$',
-        plz      => 64569,
-        word     => 'Herr',
-        age      => 55,
-    );
-    
-    for my $field ( keys %map ){
-        print "ok: ",$map{$field},"\n" if $foo->check( $field, $map{$field} );
-    }
-
-data.yml
-
-  ---
-  step1:
-      name:
-          type: required
-          length: 8,122
-      password:
-          type: required
-          length: 10,
-      plz:
-          regex: ^\d{4,5}$
-          type: optional
-      word:
-          enum:
-              - Herr
-              - Frau
-              - Firma
-      age:
-          type: required
-          min: 18
-          max: 65
-
-=head1 METHODS
-
-=head2 new
-
-  my $foo = Data::Validate::WithYAML->new( 'filename' );
-
-creates a new object.
-
-=head2 set_optional
-
-This method makes a field optional if it was required
-
-=head2 set_required
-
-This method makes a field required if it was optional
-
-=head2 validate
-
-This subroutine validates one form. You have to pass the form name (key in the
-config file), a hash with fieldnames and its values
-
-    my %fields = (
-        username => $cgi->param('user'),
-        passwort => $password,
-    );
-    $foo->validate( 'step1', %fields );
-
-=head2 fieldnames
-
-=head2 errstr
-
-=head2 message
-
-returns the message if specified in YAML
-
-  $obj->message( 'fieldname' );
-
-=head2 check_list
-
-  $obj->check_list('fieldname',['value','value2']);
-
-Checks if the values match the validation criteria. Returns an arrayref with
-checkresults:
-
-    [
-        1,
-        0,
-    ] 
-
-=head2 check
-
-  $obj->check('fieldname','value');
-
-checks if a value is valid. returns 1 if the value is valid, otherwise it
-returns 0.
-
-=head1 AUTHOR
-
-Renee Baecker, C<< <module at renee-baecker.de> >>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to
-C<bug-data-validate-withyaml at rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Data::Validate::WithYAML>.
-I will be notified, and then you'll automatically be notified of progress on
-your bug as I make changes.
-
-=head1 SUPPORT
-
-You can find documentation for this module with the perldoc command.
-
-    perldoc Data::Validate::WithYAML
-
-You can also look for information at:
-
-=over 4
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/Data::Validate::WithYAML>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/Data::Validate::WithYAML>
-
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Data::Validate::WithYAML>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/Data::Validate::WithYAML>
-
-=back
-
-=head1 ACKNOWLEDGEMENTS
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2007  - 2012 Renee Baecker, all rights reserved.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of Artistic License 2.0.
-
-=head1 AUTHOR
-
-Renee Baecker <module@renee-baecker.de>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is Copyright (c) 2012 by Renee Baecker.
-
-This is free software, licensed under:
-
-  The Artistic License 2.0 (GPL Compatible)
-
-=cut
-
+1;
